from __future__ import unicode_literals

from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from mezzanine.conf import register_setting

TOKENS = ((r"{{ *year *}}", str(timezone.now().year)),)

register_setting(
    name="MEZZY_MICROTEMPLATE_TOKENS",
    description=_(
        "Python path to a list of tokens to be replaced by Mezzy's "
        "microtemplate filter"
    ),
    editable=False,
    default="mezzy.defaults.TOKENS",
)



from __future__ import unicode_literals

from django.contrib import admin, messages
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseRedirect
from django.utils.encoding import force_text
from django.utils.html import format_html

from mezzanine.core.admin import TabularDynamicInlineAdmin
from mezzanine.utils.urls import admin_url


class LinkedAdminMixin(admin.ModelAdmin):
    """
    Admin mixin class for models that are edited only via links in a parent model.
    Links can be generated with the LinkedInlineMixin class.
    """

    readonly_fields = ["get_parent_link"]

    def __init__(self, *args, **kwargs):
        """
        Check that the parent model is specified.
        """
        super(LinkedAdminMixin, self).__init__(*args, **kwargs)
        if not hasattr(self, "parent_field"):
            raise ImproperlyConfigured(
                "You must define a 'parent_field' attribute to use LinkedAdminMixin."
            )

    def has_add_permission(self, request):
        """
        All additions should be done via the parent model.
        """
        return False

    def has_delete_permission(self, request, obj=None):
        """
        All deletions should be done via the parent model.
        """
        return False

    def has_module_permission(self, request):
        """
        Hides this admin class from the menu as this will only be accessible via the
        parent model.
        """
        return False

    def response_change(self, request, obj):
        """
        Return to the parent model instead of showing the change list view.
        """
        opts = self.model._meta
        msg_dict = {"name": force_text(opts.verbose_name), "obj": force_text(obj)}

        # Stay in the same page if the user clicks "Save and continue editing"
        if "_continue" in request.POST:
            msg = (
                "The %(name)s '%(obj)s' was changed successfully. You may edit it "
                "again below." % msg_dict
            )
            redirect_url = request.path

        # Redirect to the parent model otherwise
        else:
            parent = getattr(obj, self.parent_field)
            msg = "The %(name)s '%(obj)s' was changed successfully." % msg_dict
            redirect_url = admin_url(parent.__class__, "change", parent.pk)

        self.message_user(request, msg, messages.SUCCESS)
        return HttpResponseRedirect(redirect_url)

    def get_parent_link(self, obj):
        """
        Render a link to quickly jump to the parent model instance.
        """
        parent = getattr(obj, self.parent_field)
        parent_class = parent._meta.verbose_name.title()
        url = admin_url(parent.__class__, "change", parent.pk)
        return format_html("{}: <a href='{}'>{}</a>", parent_class, url, parent)

    get_parent_link.short_description = "Parent element"


class LinkedInlineMixin(TabularDynamicInlineAdmin):
    """
    Simple mixin to add a link to edit a related child model.
    """

    fields = ["title", "get_related_count", "get_edit_link", "_order"]
    readonly_fields = ["get_edit_link", "get_related_count"]

    def get_edit_link(self, obj):
        """
        Display a link to edit the related admin element.
        """
        if obj.id:
            txt = getattr(self, "link_text", "Edit")
            url = admin_url(obj.__class__, "change", obj.id)
            return format_html("<a href='{}'>{}</a>", url, txt)
        return ""

    get_edit_link.short_description = ""

    def get_related_count(self, obj):
        """
        Display how many related objects are available in the linked model.
        """
        if obj.id and getattr(self, "count_field", False):
            rel_field = getattr(obj, self.count_field)
            rel_name = rel_field.model()._meta.verbose_name_plural.title()
            # Sample output: 5 foobars
            return "%s %s" % (rel_field.count(), rel_name)
        return ""

    get_related_count.short_description = ""



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.views import redirect_to_login
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.utils import six
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _


#################
# Form Messages #
#################


class FormSuccessMessageMixin(object):
    """
    Add a message when form validation succeeds (requires the message framework).
    Notice that this mixin will handle both single and multi-form views,
    like the ones provided by django-extra-views.
    """

    success_message = _("All changes have been saved successfully.")

    def get_success_message(self, form=None, inlines=None):
        return self.success_message

    def form_valid(self, form):
        """
        Add the success message to a single-form view.
        """
        response = super(FormSuccessMessageMixin, self).form_valid(form)
        success_message = self.get_success_message(form)
        if success_message:
            messages.success(self.request, success_message, fail_silently=True)
        return response

    def forms_valid(self, form, inlines):
        """
        Add the success message to a form + inlines view.
        """
        response = super(FormSuccessMessageMixin, self).forms_valid(form, inlines)
        success_message = self.get_success_message(form, inlines)
        if success_message:
            messages.success(self.request, success_message, fail_silently=True)
        return response


class FormErrorMessageMixin(object):
    """
    Add a message when form validation fails (requires the message framework).
    Notice that this mixin will handle both single and multi-form views,
    like the ones provided by django-extra-views.
    """

    error_message = _("Please correct the errors below.")

    def get_error_message(self, form=None, inlines=None):
        return self.error_message

    def form_invalid(self, form):
        """
        Add the error message to a single-form view.
        """
        response = super(FormErrorMessageMixin, self).form_invalid(form)
        error_message = self.get_error_message(form)
        if error_message:
            messages.error(self.request, error_message, fail_silently=True)
        return response

    def forms_invalid(self, form, inlines):
        """
        Add the error message to a form + inlines view.
        """
        response = super(FormErrorMessageMixin, self).forms_invalid(form, inlines)
        error_message = self.get_error_message(form, inlines)
        if error_message:
            messages.error(self.request, error_message, fail_silently=True)
        return response


class FormMessagesMixin(FormSuccessMessageMixin, FormErrorMessageMixin):
    """
    Add messages when form validation succeeds or fails.
    Notice that this mixin will handle both single and multi-form views,
    like the ones provided by django-extra-views.
    """

    pass


###############################################
# Access Mixins (backported from Django 1.11) #
###############################################


class AccessMixin(object):
    """
    Abstract CBV mixin that gives access mixins the same customizable
    functionality.
    """

    login_url = None
    permission_denied_message = ""
    raise_exception = False
    redirect_field_name = REDIRECT_FIELD_NAME

    def get_login_url(self):
        """
        Override this method to override the login_url attribute.
        """
        login_url = self.login_url or settings.LOGIN_URL
        if not login_url:
            raise ImproperlyConfigured(
                "{0} is missing the login_url attribute. Define {0}.login_url, settings.LOGIN_URL, or override "  # noqa
                "{0}.get_login_url().".format(self.__class__.__name__)
            )
        return force_text(login_url)

    def get_permission_denied_message(self):
        """
        Override this method to override the permission_denied_message attribute.
        """
        return self.permission_denied_message

    def get_redirect_field_name(self):
        """
        Override this method to override the redirect_field_name attribute.
        """
        return self.redirect_field_name

    def handle_no_permission(self):
        if self.raise_exception:
            raise PermissionDenied(self.get_permission_denied_message())
        return redirect_to_login(
            self.request.get_full_path(),
            self.get_login_url(),
            self.get_redirect_field_name(),
        )  # noqa


class LoginRequiredMixin(AccessMixin):
    """
    CBV mixin which verifies that the current user is authenticated.
    """

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated():
            return self.handle_no_permission()
        return super(LoginRequiredMixin, self).dispatch(request, *args, **kwargs)


class PermissionRequiredMixin(AccessMixin):
    """
    CBV mixin which verifies that the current user has all specified
    permissions.
    """

    permission_required = None

    def get_permission_required(self):
        """
        Override this method to override the permission_required attribute.
        Must return an iterable.
        """
        if self.permission_required is None:
            raise ImproperlyConfigured(
                "{0} is missing the permission_required attribute. Define {0}.permission_required, or override "  # noqa
                "{0}.get_permission_required().".format(self.__class__.__name__)
            )
        if isinstance(self.permission_required, six.string_types):
            perms = (self.permission_required,)
        else:
            perms = self.permission_required
        return perms

    def has_permission(self):
        """
        Override this method to customize the way permissions are checked.
        """
        perms = self.get_permission_required()
        return self.request.user.has_perms(perms)

    def dispatch(self, request, *args, **kwargs):
        if not self.has_permission():
            return self.handle_no_permission()
        return super(PermissionRequiredMixin, self).dispatch(request, *args, **kwargs)


class UserPassesTestMixin(AccessMixin):
    """
    CBV Mixin that allows you to define a test function which must return True
    if the current user can access the view.
    """

    def test_func(self):
        raise NotImplementedError(
            "{0} is missing the implementation of the test_func() method.".format(
                self.__class__.__name__
            )  # noqa
        )

    def get_test_func(self):
        """
        Override this method to use a different test_func method.
        """
        return self.test_func

    def dispatch(self, request, *args, **kwargs):
        user_test_result = self.get_test_func()()
        if not user_test_result:
            return self.handle_no_permission()
        return super(UserPassesTestMixin, self).dispatch(request, *args, **kwargs)



from mezzanine.utils.importing import import_dotted_path


def import_view(dotted_path):
    """
    Import a class or function based view by its dotted path.
    """
    obj = import_dotted_path(dotted_path)
    try:
        return obj.as_view()
    except AttributeError:
        return obj



from __future__ import absolute_import, unicode_literals

from functools import update_wrapper

from django import VERSION
from django.utils.decorators import method_decorator as django_method_decorator


def method_decorator_backport(decorator, name=""):
    """
    Backport of django.utils.decorators.method_decorator from Django 1.11.
    Allows decorating classes directly, instead of having to redefine the method.
    """
    # 'obj' can be a class or a function. If 'obj' is a function at the time it
    # is passed to _dec,  it will eventually be a method of the class it is
    # defined on. If 'obj' is a class, the 'name' is required to be the name
    # of the method that will be decorated.
    from django.utils.decorators import available_attrs

    def _dec(obj):
        is_class = isinstance(obj, type)
        if is_class:
            if name and hasattr(obj, name):
                func = getattr(obj, name)
                if not callable(func):
                    raise TypeError(
                        "Cannot decorate '{0}' as it isn't a callable "
                        "attribute of {1} ({2})".format(name, obj, func)
                    )
            else:
                raise ValueError(
                    "The keyword argument `name` must be the name of a method "
                    "of the decorated class: {0}. Got '{1}' instead".format(obj, name)
                )
        else:
            func = obj

        def decorate(function):
            """
            Apply a list/tuple of decorators if decorator is one. Decorator
            functions are applied so that the call order is the same as the
            order in which they appear in the iterable.
            """
            if hasattr(decorator, "__iter__"):
                for dec in decorator[::-1]:
                    function = dec(function)
                return function
            return decorator(function)

        def _wrapper(self, *args, **kwargs):
            @decorate
            def bound_func(*args2, **kwargs2):
                return func.__get__(self, type(self))(*args2, **kwargs2)

            # bound_func has the signature that 'decorator' expects i.e.  no
            # 'self' argument, but it is a closure over self so it can call
            # 'func' correctly.
            return bound_func(*args, **kwargs)

        # In case 'decorator' adds attributes to the function it decorates, we
        # want to copy those. We don't have access to bound_func in this scope,
        # but we can cheat by using it on a dummy function.

        @decorate
        def dummy(*args, **kwargs):
            pass

        update_wrapper(_wrapper, dummy)
        # Need to preserve any existing attributes of 'func', including the name.
        update_wrapper(_wrapper, func)

        if is_class:
            setattr(obj, name, _wrapper)
            return obj

        return _wrapper

    # Don't worry about making _dec look similar to a list/tuple as it's rather
    # meaningless.
    if not hasattr(decorator, "__iter__"):
        update_wrapper(_dec, decorator, assigned=available_attrs(decorator))
    # Change the name to aid debugging.
    if hasattr(decorator, "__name__"):
        _dec.__name__ = ("method_decorator(%s)" % decorator.__name__).encode("utf-8")
    else:
        _dec.__name__ = ("method_decorator(%s)" % decorator.__class__.__name__).encode(
            "utf-8"
        )
    return _dec


if VERSION < (1, 9):
    method_decorator = method_decorator_backport
else:
    method_decorator = django_method_decorator
